


__version__ = "0.2022.3.12"



from .DirEntryX import DirEntryX
from .DescendFilter import DescendFilter
from .EmitFilter import EmitFilter
from .StdEmitFilter import StdEmitFilter

from .DirWalker import DirWalker



def scandir(
		dirPath:str,
		*,
		emitFilter:EmitFilter = None,
		descendFilter:DescendFilter = None,
	):
	yield from DirWalker(
		emitFilter=emitFilter,
		descendFilter=descendFilter,
	).scandir(dirPath)
#

def listdir(
		dirPath:str,
		*,
		emitFilter:EmitFilter = None,
		descendFilter:DescendFilter = None,
	):
	yield from DirWalker(
		emitFilter=emitFilter,
		descendFilter=descendFilter,
	).listdir(dirPath)
#









import os
import stat
import typing

import jk_typing
import jk_prettyprintobj








@typing.final
class DirEntryX(jk_prettyprintobj.DumpMixin):

	################################################################################################################################
	## Constructor
	################################################################################################################################

	#
	# Constructor method.
	#
	# @param	str absBaseDirPath		The base directory of the scanning process
	# @param	str relFilePath			The path of the element relative to the base directory path.
	# @param	os.DirEntry fe			The file entry object.
	#
	@jk_typing.checkFunctionSignature()
	def __init__(self,
			nLevel:int,
			absBaseDirPath:str,
			relPath:str,
			fileName:str,
			absPath:str,
			statResult:os.stat_result,
			exception:Exception = None,
		):

		self.absBaseDirPath = absBaseDirPath
		self.nLevel = nLevel
		self.exception = exception
		self.__statResult = statResult
		self.name = fileName
		self.fileName = fileName
		self.path = absPath
		self.absPath = absPath
		self.relPath = relPath
	#

	################################################################################################################################
	## Public Properties
	################################################################################################################################

	@property
	def isError(self) -> bool:
		return bool(self.exception)
	#

	@property
	def isWalkRoot(self) -> bool:
		return not self.relPath
	#

	@property
	def ctime(self) -> float:
		return self.__statResult.st_ctime
	#

	@property
	def mtime(self) -> float:
		return self.__statResult.st_mtime
	#

	@property
	def uid(self) -> int:
		return self.__statResult.st_uid
	#

	@property
	def gid(self) -> int:
		return self.__statResult.st_gid
	#

	@property
	def size(self) -> int:
		return self.__statResult.st_size
	#

	@property
	def isFile(self) -> bool:
		return stat.S_ISREG(self.__statResult.st_mode)
	#

	@property
	def isDir(self) -> bool:
		return stat.S_ISDIR(self.__statResult.st_mode)
	#

	@property
	def isSymLink(self) -> bool:
		return stat.S_ISLNK(self.__statResult.st_mode)
	#

	@property
	def isBlockDev(self) -> bool:
		return stat.S_ISBLK(self.__statResult.st_mode)
	#

	@property
	def isFIFO(self) -> bool:
		return stat.S_ISFIFO(self.__statResult.st_mode)
	#

	@property
	def isCharDev(self) -> bool:
		return stat.S_ISCHR(self.__statResult.st_mode)
	#

	@property
	def isSock(self) -> bool:
		return stat.S_ISSOCK(self.__statResult.st_mode)
	#

	@property
	def isOther(self) -> bool:
		m = self.__statResult.st_mode
		return not stat.S_ISREG(m) \
			and not stat.S_ISDIR(m) \
			and not stat.S_ISLNK(m) \
			and not stat.S_ISBLK(m) \
			and not stat.S_ISFIFO(m) \
			and not stat.S_ISSOCK(m) \
			and not stat.S_ISCHR(m)
	#

	@property
	def typeStr(self) -> str:
		m = self.__statResult.st_mode
		if stat.S_ISDIR(m):
			return "dir"
		elif stat.S_ISREG(m):
			return "file"
		elif stat.S_ISLNK(m):
			return "symlink"
		elif stat.S_ISBLK(m):
			return "blockdev"
		elif stat.S_ISSOCK(m):
			return "sock"
		elif stat.S_ISFIFO(m):
			return "fifo"
		elif stat.S_ISCHR(m):
			return "chardev"
		else:
			return "other"
	#

	################################################################################################################################
	## Helper Methods
	################################################################################################################################

	def _dumpVarNames(self) -> list:
		return [
			"absFilePath",
			"absBaseDirPath",
			"relFilePath",
			"fileName",
			"typeStr",
			"nLevel",
			"size",
			"uid",
			"gid",
			"ctime",
			"mtime",
			"isFile",
			"isDir",
			"isSymLink",
			"isBlockDev",
			"isCharDev",
			"isFIFO",
			"isSock",
			"isOther",
			"exception",
		]
	#

	################################################################################################################################
	## Public Methods
	################################################################################################################################

	def __repr__(self):
		if self.exception:
			return self.__class__.__name__ + "<( ERROR {}, {}, {} )>".format(self.typeStr, repr(self.relPath), repr(self.exception))
		else:
			return self.__class__.__name__ + "<( {:>8}, {} )>".format(self.typeStr, repr(self.relPath))
	#

	def __str__(self):
		return self.relPath
	#

	def stat(self) -> os.stat_result:
		return self.__statResult
	#

	def is_file(self) -> bool:
		return stat.S_ISREG(self.__statResult.st_mode)
	#

	def is_dir(self) -> bool:
		return stat.S_ISDIR(self.__statResult.st_mode)
	#

	def is_symlink(self) -> bool:
		return stat.S_ISLNK(self.__statResult.st_mode)
	#

	def is_blockDev(self) -> bool:
		return stat.S_ISBLK(self.__statResult.st_mode)
	#

	def is_fifo(self) -> bool:
		return stat.S_ISFIFO(self.__statResult.st_mode)
	#

	def is_charDev(self) -> bool:
		return stat.S_ISCHR(self.__statResult.st_mode)
	#

	def is_sock(self) -> bool:
		return stat.S_ISSOCK(self.__statResult.st_mode)
	#

	def is_other(self) -> bool:
		m = self.__statResult.st_mode
		return not stat.S_ISREG(m) \
			and not stat.S_ISDIR(m) \
			and not stat.S_ISLNK(m) \
			and not stat.S_ISBLK(m) \
			and not stat.S_ISFIFO(m) \
			and not stat.S_ISSOCK(m) \
			and not stat.S_ISCHR(m)
	#

	################################################################################################################################
	## Public Static Methods
	################################################################################################################################

	@staticmethod
	def fromOSDirEntry(nLevel:int, absBaseDirPath:str, relPath:str, fe:os.DirEntry, exception:Exception = None):
		return DirEntryX(
			nLevel = nLevel,
			absBaseDirPath = absBaseDirPath,
			relPath = relPath,
			fileName = fe.name,
			absPath = fe.path,
			statResult = fe.stat(follow_symlinks=False),
			exception = exception,
		)
	#

	def fromPath(nLevel:int, absBaseDirPath:str, absFilePath:str):
		if not absBaseDirPath.endswith(os.sep):
			_baseDirPath2 = absBaseDirPath + os.sep
		else:
			_baseDirPath2 = absBaseDirPath

		return DirEntryX(
			nLevel = nLevel,
			absBaseDirPath = absBaseDirPath,
			relPath = absFilePath[len(_baseDirPath2):],
			fileName = os.path.basename(absFilePath),
			absPath = absFilePath,
			statResult = os.stat(absFilePath, follow_symlinks=False),
			exception = None,
		)
	#

#








import os
import typing

import jk_typing
import jk_prettyprintobj

from .DirEntryX import DirEntryX
from ._WalkCtx import _WalkCtx
from .EmitFilter import EmitFilter
from .StdEmitFilter import StdEmitFilter
from .DescendFilter import DescendFilter






class DirWalker(jk_prettyprintobj.DumpMixin):

	################################################################################################################################
	## Constructor
	################################################################################################################################

	#
	# Constructor method.
	#
	@jk_typing.checkFunctionSignature()
	def __init__(self,
			*args,
			emitFilter:typing.Union[EmitFilter,typing.Callable[[DirEntryX],bool]] = None,
			descendFilter:typing.Union[DescendFilter,typing.Callable[[typing.Dict[str,DirEntryX],DirEntryX],bool]] = None,
			raiseErrors:bool = True,
		):

		assert not args

		# ----

		self.__raiseErrors = raiseErrors

		# ----

		if emitFilter is None:
			self.__emitFilter = StdEmitFilter()
			self.__emitFilterCallback = self.__emitFilter.checkEmit
		elif isinstance(emitFilter, EmitFilter):
			self.__emitFilter = emitFilter
			self.__emitFilterCallback = emitFilter.checkEmit
		elif callable(emitFilter):
			self.__emitFilter = EmitFilter()
			self.__emitFilterCallback = emitFilter
		else:
			raise Exception()

		# ----

		if descendFilter is None:
			self.__descendFilter = DescendFilter()
			self.__descendFilterCallback = self.__descendFilter.checkDescend
		elif isinstance(descendFilter, DescendFilter):
			self.__descendFilter = descendFilter
			self.__descendFilterCallback = descendFilter.checkDescend
		elif callable(descendFilter):
			self.__descendFilter = None
			self.__descendFilterCallback = descendFilter
		else:
			raise Exception()
	#

	################################################################################################################################
	## Public Properties
	################################################################################################################################

	@property
	def emitFilter(self) -> typing.Union[EmitFilter,None]:
		return self.__emitFilter
	#

	@property
	def emitFilterCallable(self) -> typing.Callable[[DirEntryX],bool]:
		return self.__emitFilterCallback
	#

	@property
	def descendFilter(self) -> typing.Union[DescendFilter,None]:
		return self.__descendFilter
	#

	@property
	def descendFilterCallable(self) -> typing.Callable[[typing.Dict[str,DirEntryX],DirEntryX],bool]:
		return self.__descendFilterCallback
	#

	@property
	def raiseErrors(self) -> bool:
		return self.__raiseErrors
	#

	################################################################################################################################
	## Helper Methods
	################################################################################################################################

	def _dumpVarNames(self) -> list:
		return [
			"descendFilter",
			"emitFilter",
			"descendFilterCallable",
			"emitFilterCallable",
			"raiseErrors",
		]
	#

	def __walk0(
			self,
			ctx:_WalkCtx,
			nLevel:int,
			absWalkDirPath:str,
			relWalkDirPath:str,
			parentDirEntry:DirEntryX,
		) -> typing.Iterable[os.DirEntry]:

		assert isinstance(ctx, _WalkCtx)
		assert isinstance(absWalkDirPath, str)
		assert isinstance(relWalkDirPath, str)
		assert isinstance(parentDirEntry, DirEntryX)

		# ----------------------------------------------------------------

		if self.__raiseErrors:
			allEntries = list(os.scandir(absWalkDirPath))
		else:
			try:
				allEntries = list(os.scandir(absWalkDirPath))
			except PermissionError as ee:
				parentDirEntry.exception = ee
				if self.__emitFilter.emitErrors:
					yield parentDirEntry
				return

		# ----------------------------------------------------------------
		# create all DirEntryX elements and store them in a map

		allEntriesMap = {}
		for fe in allEntries:
			assert isinstance(fe, os.DirEntry)
			relPath = os.path.join(relWalkDirPath, fe.name)
			_entry = DirEntryX.fromOSDirEntry(nLevel, ctx.baseDirPath, relPath, fe)
			allEntriesMap[fe.name] = _entry

		# ----------------------------------------------------------------

		for fileEntryName in sorted(allEntriesMap.keys()):
			feX = allEntriesMap[fileEntryName]
			assert isinstance(feX, DirEntryX)

			if self.__emitFilterCallback(feX):
				yield feX

			if feX.is_dir():
				if self.__descendFilterCallback(allEntriesMap, feX):
					yield from self.__walk0(ctx, nLevel+1, feX.absPath, feX.relPath, feX)
	#

	################################################################################################################################
	## Public Methods
	################################################################################################################################

	def listdir(self, dirPath:str) -> typing.Iterable[str]:
		for x in self.scandir(dirPath):
			yield x.relPath
	#

	def scandir(self, dirPath:str) -> typing.Iterable[DirEntryX]:
		assert isinstance(dirPath, str)
		dirPath = os.path.expanduser(dirPath)
		assert os.path.isdir(dirPath)

		dirPath = os.path.abspath(dirPath)

		# ----

		subEntry = DirEntryX.fromPath(-1, dirPath, dirPath)
		if self.__emitFilter.emitWalkRoot:
			yield subEntry

		yield from self.__walk0(
			_WalkCtx(dirPath),
			0,
			dirPath,
			"",
			subEntry,
		)
	#

#










import os
import typing

import jk_typing

from .DirEntryX import DirEntryX
from .EmitFilter import EmitFilter






class StdEmitFilter(EmitFilter):

	################################################################################################################################
	## Constructor
	################################################################################################################################

	@jk_typing.checkFunctionSignature()
	def __init__(self,
			emitWalkRoot:bool = True,
			emitDirectories:bool = True,
			emitRegularFiles:bool = True,
			emitSymLinks:bool = True,
			emitBlockDevices:bool = True,
			emitCharacterDevices:bool = True,
			emitSockets:bool = True,
			emitFIFOs:bool = True,
			emitOthers:bool = True,
			emitErrors:bool = True,
		) -> None:

		super().__init__(
			emitWalkRoot = emitWalkRoot,
			emitErrors = emitErrors,
		)

		self.emitDirectories = emitDirectories
		self.emitRegularFiles = emitRegularFiles
		self.emitSymLinks = emitSymLinks
		self.emitBlockDevices = emitBlockDevices
		self.emitCharacterDevices = emitCharacterDevices
		self.emitSockets = emitSockets
		self.emitFIFOs = emitFIFOs
		self.emitOthers = emitOthers
	#

	################################################################################################################################
	## Public Properties
	################################################################################################################################

	################################################################################################################################
	## Helper Methods
	################################################################################################################################

	def _dumpVarNames(self) -> list:
		return [
			"emitWalkRoot",
			"emitDirectories",
			"emitRegularFiles",
			"emitSymLinks",
			"emitBlockDevices",
			"emitCharacterDevices",
			"emitSockets",
			"emitFIFOs",
			"emitOthers",
			"emitErrors",
		]
	#

	################################################################################################################################
	## Public Methods
	################################################################################################################################

	def checkEmit(self, entry:DirEntryX) -> bool:
		if entry.is_dir():
			if self.emitDirectories:
				return entry
		elif entry.is_file():
			if self.emitRegularFiles:
				return entry
		elif entry.is_symlink():
			if self.emitSymLinks:
				return entry
		elif entry.is_blockDev():
			if self.emitBlockDevices:
				return entry
		elif entry.is_charDev():
			if self.emitCharacterDevices:
				return entry
		elif entry.is_sock():
			if self.emitSockets:
				return entry
		elif entry.is_fifo():
			if self.emitFIFOs:
				return entry
		elif entry.is_other():
			if self.emitOthers:
				return entry
		else:
			entry.dump()
			raise Exception()
	#

	@staticmethod
	def newFromDisabled(
			emitWalkRoot:bool = False,
			emitDirectories:bool = False,
			emitRegularFiles:bool = False,
			emitSymLinks:bool = False,
			emitBlockDevices:bool = False,
			emitCharacterDevices:bool = False,
			emitSockets:bool = False,
			emitFIFOs:bool = False,
			emitOthers:bool = False,
			emitErrors:bool = False,
		):

		return StdEmitFilter(
			emitWalkRoot = emitWalkRoot,
			emitDirectories = emitDirectories,
			emitRegularFiles = emitRegularFiles,
			emitSymLinks = emitSymLinks,
			emitBlockDevices = emitBlockDevices,
			emitCharacterDevices = emitCharacterDevices,
			emitSockets = emitSockets,
			emitFIFOs = emitFIFOs,
			emitOthers = emitOthers,
			emitErrors = emitErrors,
		)
	#

#







